// SPDX-FileCopyrightText: 2022 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#ifndef UT_PROEXPRESSIONBAR_H
#define UT_PROEXPRESSIONBAR_H

#include <gtest/gtest.h>
#include <gmock/gmock-matchers.h>
#include <QTest>

#include "../../src/dsettings.h"

class Ut_ProexpressionBar : public ::testing::Test
{
public:
    Ut_ProexpressionBar();
};

#endif // UT_PROEXPRESSIONBAR_H
