// SPDX-FileCopyrightText: 2022 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#ifndef UT_MEMHISWIDGET_H
#define UT_MEMHISWIDGET_H

#include <gtest/gtest.h>
#include <gmock/gmock-matchers.h>
#include <QTest>

class Ut_MemHisWidget : public ::testing::Test
{
public:
    Ut_MemHisWidget();
};

#endif // UT_MEMHISWIDGET_H
