// SPDX-FileCopyrightText: 2022 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#ifndef UT_INPUTEDIT_H
#define UT_INPUTEDIT_H

#include <gtest/gtest.h>
#include <gmock/gmock-matchers.h>
#include <QTest>
#include <QMenu>

#include"../stub.h"

class Ut_InputEdit : public ::testing::Test
{
public:
    Ut_InputEdit();
};

#endif // UT_INPUTEDIT_H
