// SPDX-FileCopyrightText: 2022 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#ifndef GLOBALDEFINE_H
#define GLOBALDEFINE_H

//中文符号

#define CN_ADD QString::fromUtf8("＋")
#define CN_MIN QString::fromUtf8("－")
#define CN_MUL QString::fromUtf8("×")
#define CN_Asterisk QString::fromUtf8("＊")
#define CN_DIV QString::fromUtf8("÷")
#define CN_LBracket QString::fromUtf8("（")
#define CN_RBracket QString::fromUtf8("）")
#define CN_Period QString::fromUtf8("。")
#define CN_Underscore QString::fromUtf8("——")
#define CN_Percent QString::fromUtf8("％")

//英文符号
#define EN_Comma QString::fromUtf8(",")
#define EN_PI QString::fromUtf8("π")
#define EN_Eular QString::fromUtf8("e")
#define EN_ADD QString("+")
#define EN_MIN QString("-")
#define EN_MUL QString("*")
#define EN_DIV QString("/")
#define EN_LBracket QString("(")
#define EN_RBracket QString(")")
#define EN_Percent QString("%")
#define EN_Underscore QString("_")
#define EN_Period QString(".")

#endif // GLOBALDEFINE_H
